# coding=utf-8
#
# MIT License
#
# Copyright (c) 2022 KuFlow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from datetime import date, datetime
from typing import Any, Dict, Optional

from ..models import (
    JsonFormsFile,
    JsonFormsPrincipal,
    TaskSaveJsonFormsValueDataCommand,
)
from .json_forms import (
    JsonFormDataAccessor,
    JsonFormsSimpleType,
    find_json_forms_property_as_date,
    find_json_forms_property_as_datetime,
    find_json_forms_property_as_dict,
    find_json_forms_property_as_float,
    find_json_forms_property_as_int,
    find_json_forms_property_as_json_forms_file,
    find_json_forms_property_as_json_forms_principal,
    find_json_forms_property_as_list,
    find_json_forms_property_as_str,
    get_json_forms_property_as_date,
    get_json_forms_property_as_datetime,
    get_json_forms_property_as_dict,
    get_json_forms_property_as_float,
    get_json_forms_property_as_int,
    get_json_forms_property_as_json_forms_file,
    get_json_forms_property_as_json_forms_principal,
    get_json_forms_property_as_list,
    get_json_forms_property_as_str,
    update_json_forms_property,
)


class CurrentJsonFormDataAccessor(JsonFormDataAccessor):
    def __init__(self, command: TaskSaveJsonFormsValueDataCommand):
        self.command = command

    def get_data(self) -> Optional[Dict[str, Any]]:
        return self.command.data

    def set_data(self, data: Optional[Dict[str, Any]]):
        self.command.data = data


class TaskSaveJsonFormsValueDataCommandUtils:
    @staticmethod
    def get_json_forms_property_as_str(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> str:
        """
        Get a json property as "str" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_str(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_str(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[str]:
        """
        Try to find a json property as "str" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_str(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_int(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> int:
        """
        Get a json property as "int" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_int(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_int(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[int]:
        """
        Try to find a json property as "int" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_int(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_float(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> float:
        """
        Get a json property as "float" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_float(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_float(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[float]:
        """
        Try to find a json property as "float" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_float(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_date(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> date:
        """
        Get a json property as "date" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_date(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_date(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[date]:
        """
        Try to find a json property as "date" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_date(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_datetime(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> datetime:
        """
        Get a json property as "datetime" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_datetime(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_datetime(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[datetime]:
        """
        Try to find a json property as "datetime" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_datetime(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_file(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> JsonFormsFile:
        """
        Get a json property as "JsonFormsFile" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_json_forms_file(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_file(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[JsonFormsFile]:
        """
        Try to find a json property as "JsonFormsFile" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_json_forms_file(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_principal(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> JsonFormsPrincipal:
        """
        Get a json property as "JsonFormsPrincipal" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_json_forms_principal(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_principal(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[JsonFormsPrincipal]:
        """
        Try to find a json property as "JsonFormsPrincipal" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_json_forms_principal(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_list(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> list:
        """
        Get a json property as "list" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_list(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_list(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[list]:
        """
        Try to find a json property as "list" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_list(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def get_json_forms_property_as_dict(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> dict:
        """
        Get a json property as "dict" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            the property value if exists.

        Raises:
            ValueError: If property value doesn't exist or has incorrect format
        """
        return get_json_forms_property_as_dict(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def find_json_forms_property_as_dict(
        command: TaskSaveJsonFormsValueDataCommand, property_path: str
    ) -> Optional[dict]:
        """
        Try to find a json property as "dict" following the "propertyPath" passed.

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"

        Return:
            The property value if exists.

        Raises:
            ValueError: If property value has incorrect format
        """
        return find_json_forms_property_as_dict(
            CurrentJsonFormDataAccessor(command), property_path
        )

    @staticmethod
    def update_json_forms_property(
        command: TaskSaveJsonFormsValueDataCommand,
        property_path: str,
        value: Optional[JsonFormsSimpleType],
    ) -> None:
        """
        Update a json forms data property in the task passed following the "property_path".

        Arguments:
            command: The task save json forms value data command.
            property_path: Property path to find. ie: "user.name" or "users.0.name"
            value: Value to update

        Raises:
            ValueError: If property value has incorrect format
        """
        update_json_forms_property(
            CurrentJsonFormDataAccessor(command), property_path, value
        )



# coding=utf-8
#
# MIT License
#
# Copyright (c) 2022 KuFlow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

from typing import Any, List, Optional, Union

from .. import models as _models
from .._generated import KuFlowRestClient as KuFlowRestClientGenerated


class ProcessOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~kuflow.rest.client.KuFlowRestClient`'s
        :attr:`process` attribute.
    """

    def __init__(self, kuflow_client: KuFlowRestClientGenerated):
        self._kuflow_client = kuflow_client

    def find_processes(
        self,
        size: int = 25,
        page: int = 0,
        sort: Optional[Union[str, List[str]]] = None,
        **kwargs: Any,
    ) -> _models.ProcessPage:
        """Find all accessible Processes.

        List all the Processes that have been created and the credentials has access.

        Available sort query values: id, createdAt, lastModifiedAt.

        :keyword size: The number of records returned within a single API call. Default value is 25.
        :type size: int
        :keyword page: The page number of the current page in the returned records, 0 is the first page.
                       Default value is 0.
        :type page: int
        :keyword sort: Sorting criteria in the format: property{,asc|desc}. Example: createdAt,desc

                       Default sort order is ascending. Multiple sort criteria are supported.

                       Please refer to the method description for supported properties. Default value is None.
        :type sort: Optional[Union[str, List[str]]]
        :return: ProcessPage
        :rtype: ~kuflow.rest.models.ProcessPage
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        if sort is not None and isinstance(sort, str):
            sort = [sort]

        return self._kuflow_client.process.find_processes(
            size=size, page=page, sort=sort, **kwargs
        )

    def create_process(
        self, process: _models.Process, **kwargs: Any
    ) -> _models.Process:
        """Create a new process.

        Creates a process. This option has direct correspondence to the action of starting a process in
        the Kuflow GUI.

        When a process is created, the current user is assigned as the process initiator, if you want
        to change it, you can pass a valid initiator using the following options:


        * If you know the ``principal ID`` you can assign it to ``initiator.id``
        * If you know the ``user ID`` you can assign it to ``initiator.user.id``
        * If you know the ``user email`` you can assign it to ``initiator.user.email``
        * If you know the ``application ID`` you can assign it to ``initiator.application.id``

        If you want the method to be idempotent, please specify the ``id`` field in the request body.

        :param process: Process to create. Required.
        :type process: ~kuflow.rest.models.Process
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.create_process(process=process, **kwargs)

    def retrieve_process(self, id: str, **kwargs: Any) -> _models.Process:
        """Get a Process by ID.

        Returns the requested Process when has access to do it.

        :param id: The resource ID. Required.
        :type id: str
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.retrieve_process(id=id, **kwargs)

    def actions_process_change_initiator(
        self, id: str, command: _models.ProcessChangeInitiatorCommand, **kwargs: Any
    ) -> _models.Process:
        """Change process initiator.

        Change the current initiator of a process.

        Allows you to choose a user (by email or principal identifier) or an application (principal
        identifier).
        Only one option will be necessary.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to change the process initiator. Required.
        :type command: ~kuflow.rest.models.ProcessChangeInitiatorCommand
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.actions_process_change_initiator(
            id=id, command=command, **kwargs
        )

    def actions_process_save_element(
        self, id: str, command: _models.ProcessSaveElementCommand, **kwargs: Any
    ) -> _models.Process:
        """Save a process element, aka: metadata.

        Allow to save an element.

        If values already exist for the provided element code, it replaces them with the new ones,
        otherwise it creates them. The values of the previous elements that no longer exist will be
        deleted.

        If the process is already finished the invocations fails with an error.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to save an element. Required.
        :type command: ~kuflow.rest.models.ProcessSaveElementCommand
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.actions_process_save_element(
            id=id, command=command, **kwargs
        )

    def actions_process_delete_element(
        self, id: str, command: _models.ProcessDeleteElementCommand, **kwargs: Any
    ) -> _models.Process:
        """Delete an element by code.

        Allow to delete a process element by specifying the item definition code.

        Remove all the element values.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to delete an element. Required.
        :type command: ~kuflow.rest.models.ProcessDeleteElementCommand
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.actions_process_delete_element(
            id=id, command=command, **kwargs
        )

    def actions_process_complete(self, id: str, **kwargs: Any) -> _models.Process:
        """Complete a Process.

        Complete a Process. The state of Process is set to 'completed'.

        If you are already in this state, no action is taken.

        :param id: The resource ID. Required.
        :type id: str
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.actions_process_complete(id=id, **kwargs)

    def actions_process_cancel(self, id: str, **kwargs: Any) -> _models.Process:
        """Cancel a Process.

        Cancel a Process. The Process state is set to 'cancelled'.

        All the active tasks will be marked as cancelled too.

        If you are already in this state, no action is taken.

        :param id: The resource ID. Required.
        :type id: str
        :return: Process
        :rtype: ~kuflow.rest.models.Process
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.process.actions_process_cancel(id=id, **kwargs)

    def actions_process_save_user_action_value_document(
        self,
        id: str,
        file: _models.Document,
        command: _models.ProcessSaveUserActionValueDocumentCommand,
        **kwargs: Any,
    ) -> Optional[_models.Process]:
        """Upload and save a document in a user action.

        Allow saving a user action document uploading the content.

        :param id: The resource ID. Required.
        :type id: str
        :param file: Document to save. Required.
        :type file: _models.Document
        :keyword command: User action info. Required.
        :type command: _models.ProcessSaveUserActionValueDocumentCommand
        :return: Process or None
        :rtype: ~kuflow.rest.models.Process or None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return (
            self._kuflow_client.process.actions_process_save_user_action_value_document(
                id=id,
                file=file.file_content,
                file_content_type=file.content_type,
                file_name=file.file_mame,
                user_action_value_id=command.user_action_value_id,
                **kwargs,
            )
        )



# coding=utf-8
#
# MIT License
#
# Copyright (c) 2022 KuFlow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#


from typing import Any, Iterator, List, Optional, Union

from .. import models as _models
from .._generated import KuFlowRestClient as KuFlowRestClientGenerated


class TaskOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~kuflow.rest.client.KuFlowRestClient`'s
        :attr:`task` attribute.
    """

    def __init__(self, kuflow_client: KuFlowRestClientGenerated):
        self._kuflow_client = kuflow_client

    def find_tasks(
        self,
        size: int = 25,
        page: int = 0,
        sort: Optional[Union[str, List[str]]] = None,
        process_id: Optional[Union[str, List[str]]] = None,
        state: Optional[Union[_models.TaskState, List[_models.TaskState]]] = None,
        task_definition_code: Optional[Union[str, List[str]]] = None,
        **kwargs: Any,
    ) -> _models.TaskPage:
        """Find all accessible Tasks.

        List all Tasks that have been created and the credentials has access.

        Available sort query values: id, createdAt, lastModifiedAt, claimedAt, completedAt,
        cancelledAt.

        :keyword size: The number of records returned within a single API call. Default value is 25.
        :type size: int
        :keyword page: The page number of the current page in the returned records, 0 is the first page.
                       Default value is 0.
        :type page: int
        :keyword sort: Sorting criteria in the format: property{,asc|desc}. Example: createdAt,desc
                       Default sort order is ascending. Multiple sort criteria are supported.
                       Please refer to the method description for supported properties. Default value is None.
        :type sort: list[str]
        :keyword process_id: Filter by an array of process ids. Default value is None.
        :type process_id: list[str]
        :keyword state: Filter by an array of task states. Default value is None.
        :type state: list[str or ~kuflow.rest.models.TaskState]
        :keyword task_definition_code: Filter by an array of task definition codes. Default value is None.
        :type task_definition_code: list[str]
        :return: TaskPage
        :rtype: ~kuflow.rest.models.TaskPage
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        if sort is not None and isinstance(sort, str):
            sort = [sort]
        if process_id is not None and isinstance(process_id, str):
            process_id = [process_id]
        if state is not None and isinstance(state, _models.TaskState):
            state = [state]
        if task_definition_code is not None and isinstance(task_definition_code, str):
            task_definition_code = [task_definition_code]

        return self._kuflow_client.task.find_tasks(
            size=size,
            page=page,
            sort=sort,
            process_id=process_id,
            state=state,
            task_definition_code=task_definition_code,
            **kwargs,
        )

    def create_task(self, task: _models.Task, **kwargs: Any) -> _models.Task:
        """Create a new Task in the selected Process.

        Create a Task and optionally fill its elements. We can fill in any type of element except
        documents.

        If you want to add document type elements, you can pass a reference to an existing document
        type element indicating its 'uri'. This will copy that document into the element. In case you
        want to add a new document, please use the corresponding API method.

        If you want that the task created is claimed you can a valid owner using the following options:


        * If you know the ``principal ID`` you can assign it to ``owner.id``
        * If you know the ``user ID`` you can assign it to ``owner.user.id``
        * If you know the ``user email`` you can assign it to ``owner.user.email``
        * If you know the ``application ID`` you can assign it to ``owner.application.id``

        If you want the method to be idempotent, please specify the ``id`` field in the request body.

        :param task: Task to be created. Required.
        :type task: ~kuflow.rest.models.Task
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.create_task(task=task, **kwargs)

    def retrieve_task(self, id: str, **kwargs: Any) -> _models.Task:
        """Get a task given it ID.

        Allow to get a task by ID.

        :param id: The resource ID. Required.
        :type id: str
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.retrieve_task(id=id, **kwargs)

    def actions_task_claim(self, id: str, **kwargs: Any) -> _models.Task:
        """Claim a task.

        Allow to claim a task.

        :param id: The resource ID. Required.
        :type id: str
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_claim(id=id, **kwargs)

    def actions_task_assign(
        self, id: str, command: _models.TaskAssignCommand, **kwargs: Any
    ) -> _models.Task:
        """Assign a task.

        Allow to assign a task to a user or application. Only one option will be necessary.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to change the task owner. Required.
        :type command: ~kuflow.rest.models.TaskAssignCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_assign(
            id=id, command=command, **kwargs
        )

    def actions_task_save_element(
        self, id: str, command: _models.TaskSaveElementCommand, **kwargs: Any
    ) -> _models.Task:
        """Save an element.

        Allow to save an element i.e., a field, a decision, a form, a principal or document.

        In the case of document type elements, this method only allows references to be made to other
        existing document type elements for the purpose of copying that file into the element. To do
        this you need to pass a reference to the document using the 'uri' attribute. In case you want
        to add a new document, please use the corresponding API method. If values already exist for the
        provided element code, it replaces them with the new ones, otherwise it creates them. The
        values of the previous elements that no longer exist will be deleted. To remove an element, use
        the appropriate API method.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to save an element. Required.
        :type command: ~kuflow.rest.models.TaskSaveElementCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_save_element(
            id=id, command=command, **kwargs
        )

    def actions_task_save_element_value_document(
        self,
        id: str,
        file: _models.Document,
        command: _models.TaskSaveElementValueDocumentCommand,
        **kwargs: Any,
    ) -> _models.Task:
        """Save an element document.

        Allow to save an element document uploading the content.

        If it is a multiple element, and the ID referenced in the body does not exist or is empty, the
        document will be added to the element. If the element already exists (the ID referenced in the
        body corresponds to an existing one), it updates it.

        :param id: The resource ID. Required.
        :type id: str
        :param file: Document to save. Required.
        :type file: ~kuflow.rest.models.Document
        :param command: Command info. Required.
        :type command: ~kuflow.rest.models.TaskSaveElementValueDocumentCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_save_element_value_document(
            id=id,
            file=file.file_content,
            file_content_type=file.content_type,
            file_name=file.file_mame,
            element_definition_code=command.element_definition_code,
            element_value_id=command.element_value_id,
            element_value_valid=command.element_value_valid,
            **kwargs,
        )

    def actions_task_delete_element(
        self, id: str, command: _models.TaskDeleteElementCommand, **kwargs: Any
    ) -> _models.Task:
        """Delete an element by code.

        Allow to delete task element by specifying the item definition code.

        Remove all the element values.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to delete an element. Required.
        :type command: ~kuflow.rest.models.TaskDeleteElementCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_delete_element(
            id=id, command=command, **kwargs
        )

    def actions_task_delete_element_value_document(
        self,
        id: str,
        command: _models.TaskDeleteElementValueDocumentCommand,
        **kwargs: Any,
    ) -> _models.Task:
        """Delete an element document value.

        Allow to delete a specific document from an element of document type using its id.

        Note: If it is a multiple item, it will only delete the specified document. If it is a single
        element, in addition to the document, it will also delete the element.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to delete a document element value. Required.
        :type command: ~kuflow.rest.models.TaskDeleteElementValueDocumentCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_delete_element_value_document(
            id=id, command=command, **kwargs
        )

    def actions_task_download_element_value_document(
        self, id: str, document_id: str, **kwargs: Any
    ) -> Iterator[bytes]:
        """Download document.

        Given a task, download a document from an element of document type.

        :param id: The resource ID. Required.
        :type id: str
        :keyword document_id: Document ID to download. Required.
        :type document_id: str
        :return: Iterator of the response bytes
        :rtype: Iterator[bytes]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_download_element_value_document(
            id=id, document_id=document_id, **kwargs
        )

    def actions_task_download_element_value_rendered(
        self, id: str, element_definition_code: str, **kwargs: Any
    ) -> Iterator[bytes]:
        """Download a Form rendered as PDF or Zip of PDFs (when the element is multiple).

        Given a task, generate a PDF from a Form type element with the data filled in, if any. If there
        are multiple form values, they are packed into a ZIP.

        Important!: To use this feature, please contact to kuflow@kuflow.com.

        :param id: The resource ID. Required.
        :type id: str
        :param element_definition_code: Element definition code of a Form Element to download.
        :type element_definition_code: str
        :return: Iterator of the response bytes
        :rtype: Iterator[bytes]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_download_element_value_rendered(
            id=id, element_definition_code=element_definition_code, **kwargs
        )

    def actions_task_save_json_forms_value_data(
        self,
        id: str,
        command: _models.TaskSaveJsonFormsValueDataCommand,
        **kwargs: Any,
    ) -> _models.Task:
        """Save JSON data.

        Allow to save a JSON data validating that the data follow the related schema. If the data is
        invalid, then
        the json form is marked as invalid.

        :param id: The resource ID. Required.
        :type id: str
        :param command: Command to save the JSON value. Required.
        :type command: ~kuflow.rest.models.TaskSaveJsonFormsValueDataCommand
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_save_json_forms_value_data(
            id=id, command=command, **kwargs
        )

    def actions_task_save_json_forms_value_document(
        self,
        id: str,
        file: _models.Document,
        command: _models.TaskSaveJsonFormsValueDocumentRequestCommand,
        **kwargs: Any,
    ) -> _models.TaskSaveJsonFormsValueDocumentResponseCommand:
        """Save a JSON Forms document.

        Save a document in the task to later be linked into the JSON data.

        :param id: The resource ID. Required.
        :type id: str
        :param file: Document to save. Required.
        :type file: ~kuflow.rest.models.Document
        :param command: Command to save the JSON value. Required.
        :type command: ~kuflow.rest.models.TaskSaveJsonFormsValueDataCommand
        :return: TaskSaveJsonFormsValueDocumentResponseCommand
        :rtype: ~kuflow.rest.models.TaskSaveJsonFormsValueDocumentResponseCommand
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_save_json_forms_value_document(
            id=id,
            file=file.file_content,
            file_content_type=file.content_type,
            file_name=file.file_mame,
            schema_path=command.schema_path,
            **kwargs,
        )

    def actions_task_download_json_forms_value_document(
        self, id: str, *, document_uri: str, **kwargs: Any
    ) -> Iterator[bytes]:
        """Download document.

        Given a task, download a document from a json form data.

        :param id: The resource ID. Required.
        :type id: str
        :param document_uri: Document URI to download. Required.
        :type document_uri: str
        :return: Iterator of the response bytes
        :rtype: Iterator[bytes]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_download_json_forms_value_document(
            id=id,
            document_uri=document_uri,
            **kwargs,
        )

    def actions_task_complete(self, id: str, **kwargs: Any) -> _models.Task:
        """Complete a task.

        Allow to complete a claimed task by the principal.

        :param id: The resource ID. Required.
        :type id: str
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_complete(id=id, **kwargs)

    def actions_task_append_log(
        self, id: str, log: _models.Log, **kwargs: Any
    ) -> _models.Task:
        """Append a log to the task.

        A log entry is added to the task. If the number of log entries is reached, the oldest log entry
        is removed.

        :param id: The resource ID. Required.
        :type id: str
        :param log: Log to be created. Required.
        :type log: ~kuflow.rest.models.Log
        :return: Task
        :rtype: ~kuflow.rest.models.Task
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        return self._kuflow_client.task.actions_task_append_log(
            id=id, log=log, **kwargs
        )



# coding=utf-8
# --------------------------------------------------------------------------
#
# MIT License
#
# Copyright (c) 2022 KuFlow
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# --------------------------------------------------------------------------
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
#
# --------------------------------------------------------------------------

from typing import Any, TYPE_CHECKING

from azure.core.configuration import Configuration
from azure.core.pipeline import policies

from ._version import VERSION

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials import TokenCredential


class KuFlowRestClientConfiguration(Configuration):  # pylint: disable=too-many-instance-attributes,name-too-long
    """Configuration for KuFlowRestClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials.TokenCredential
    """

    def __init__(self, credential: "TokenCredential", **kwargs: Any) -> None:
        super(KuFlowRestClientConfiguration, self).__init__(**kwargs)
        if credential is None:
            raise ValueError("Parameter 'credential' must not be None.")

        self.credential = credential
        self.credential_scopes = kwargs.pop("credential_scopes", [])
        kwargs.setdefault("sdk_moniker", "kuflow-rest/{}".format(VERSION))
        self._configure(**kwargs)

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get(
            "user_agent_policy"
        ) or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(
            **kwargs
        )
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get(
            "logging_policy"
        ) or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get(
            "http_logging_policy"
        ) or policies.HttpLoggingPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.RetryPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get(
            "custom_hook_policy"
        ) or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.RedirectPolicy(
            **kwargs
        )
        self.authentication_policy = kwargs.get("authentication_policy")
        if not self.credential_scopes and not self.authentication_policy:
            raise ValueError(
                "You must provide either credential_scopes or authentication_policy as kwargs"
            )
        if self.credential and not self.authentication_policy:
            self.authentication_policy = policies.BearerTokenCredentialPolicy(
                self.credential, *self.credential_scopes, **kwargs
            )
